-- logic analyser (LA) for FPGAs
-- ver 1.1
-- Author: Ernest Jamro
-- Run Length Codding
-- if the MSB of the data out is 0 - data
    -- else count is written into the memory

--//////////////////////////////////////////////////////////////////////
--//// Copyright (C) 2001 Authors and OPENCORES.ORG                 ////
--////                                                              ////
--//// This source file may be used and distributed without         ////
--/// restriction provided that this copyright statement is not    ////
--//// removed from the file and that any derivative work contains  ////
--//// the original copyright notice and the associated disclaimer. ////
--////                                                              ////
--//// This source file is free software; you can redistribute it   ////
--//// and/or modify it under the terms of the GNU Lesser General   ////
--//// Public License as published by the Free Software Foundation; ////
--//// either version 2.1 of the License, or (at your option) any   ////
--//// later version.                                               ////
--////                                                              ////
--//// This source is distributed in the hope that it will be       ////
--//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
--//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
--//// PURPOSE. See the GNU Lesser General Public License for more  ////
--//// details.                                                     ////
--////                                                              ////
--//// You should have received a copy of the GNU Lesser General    ////
--//// Public License along with this source; if not, download it   ////
--//// from <http://www.opencores.org/lgpl.shtml>                   ////

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_unsigned.all;
use IEEE.std_logic_misc.all;

entity la_rlc is
  generic(dwidth: integer); -- data width
  port(clk, arst: std_logic; -- clock adn asynch reset  
     ce: in std_logic; -- clock enable
  	 stop_coding: in std_logic; -- do not use run lenght coding 
     we: out std_logic; -- memory write enable
     din: in std_logic_vector(dwidth-1 downto 0); -- data in (before compression)- the MSB is ignored
     dout: out std_logic_vector(dwidth-1 downto 0)); -- data out (after commpression)
end la_rlc;


       

architecture la_rlc of la_rlc is
  signal count: std_logic_vector(dwidth-2 downto 0); -- same data in (repeat counter)
  signal count_in: std_logic_vector(dwidth-1 downto 0); 
  signal din_q: std_logic_vector(dwidth-2 downto 0);
  signal same_data, same_data_q: std_logic; -- same data as previous
  signal same_data_vec: std_logic_vector(dwidth-2 downto 0); -- comapare each bit seperately
begin
  -- data in flip-flops
  fdin: process(arst, clk) begin
    if arst='1' then din_q<= (others=>'0'); same_data_q<= '1';
    elsif clk'event and clk='1' then
      if ce='1' then 
        din_q<= din(dwidth-2 downto 0);
        same_data_q<= same_data;
      end if;
    end if;
  end process;
  
  -- counter (how many times the same data are presented
  counter: process(arst, clk) begin
    if arst= '1' then count<= (others=>'0');
    elsif clk'event and clk='1' then
      if ce='1' then 
        if same_data_q='0' then count<= (others=>'0');
        else count<= count_in(dwidth-2 downto 0);
        end if;
      end if;
    end if;
  end process;
  
  count_in<= ('0' & count) + 1;
  
  gsi: for i in 0 to dwidth-2 generate
    same_data_vec(i)<= '1' when din(i) = din_q(i) else '0';
  end generate;
  same_data<= and_reduce(same_data_vec) and not stop_coding;
  
  we<= ce when same_data='0' or same_data_q='0' -- different data 
       or count_in(dwidth-1)='1' else '0'; -- counter overrun
  
  dout<= '0' & din_q when same_data_q='0' 
       else '1' & count;
    
end la_rlc;
